// vi: set ts=4 sw=4 :
// vim: set tw=75 :
// engine_api.cpp - implementation of Half-Life engine functions
/*
* Copyright(c)2001 Will Day <willday@hpgx.net>
*
* This file is part of Metamod.
*
* Metamod is free software; you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2 of the License, or(at
* your option)any later version.
*
* Metamod is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Metamod; if not, write to the Free Software Foundation,
* Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* In addition, as a special exception, the author gives permission to
* link the code of this program with the Half-Life Game Engine("HL
* Engine")and Modified Game Libraries("MODs")developed by Valve,
* L.L.C("Valve"). You must obey the GNU General Public License in all
* respects for all of the code used other than the HL Engine and MODs
* from Valve. If you modify this file, you may extend this exception
* to your version of the file, but you are not obligated to do so. If
* you do not wish to do so, delete this exception statement from your
* version.
*
*/
#include <stdio.h>
#include <stdarg.h>
#include <extdll.h>
#include "engine_api.h"
#include "meta_api.h"
#include "sdk_util.h"
#include "progdefs.h"
#include "edict.h"
#include "const.h"
#include "cbase.h"
#include "enginecallback.h"
#include "weaponinfo.h"
#include "progdefs.h"
#include "scriplib.h"
#include "bmx_plugin.h"
#include "cstrike.h"

#include "item.h"
#include "usrmsg.h"
#include "playerbuy.h"
#include "moneymsg.h"
extern char *g_argv;
int isFakeClientCommand=0;
int fake_arg_count;

void SetModel(edict_t *e, const char *m)
{
	if(g_mod_type==CSTRIKE)CS_SetModel(e, m);
	RETURN_META(MRES_IGNORED);
}
edict_t* FindEntityByString(edict_t *pEdictStartSearchAfter, const char *pszField, const char *pszValue)
{
	bSpawn=false;
	if(strcmp(pszValue, "info_map_parameters")==0)
	{
		if(g_mod_type==CSTRIKE)CS_NewRound();
	}
	RETURN_META_VALUE(MRES_IGNORED,(edict_t*)META_RESULT_PREVIOUS);
}
void EmitSound(edict_t *entity, int channel, const char *sample, float volume, float attenuation, int fFlags, int pitch)
{
	if(g_mod_type==CSTRIKE&&CS_EmitSound(entity, channel, sample, volume, attenuation, fFlags, pitch)) {
		RETURN_META(MRES_SUPERCEDE);
	}
	else {
		RETURN_META(MRES_IGNORED);
	}
}

void MessageEnd_Post(void)
{
	g_msg_type=MSGEND;
	g_msg_index=MSGEND;
	MsgFunc(0);
	MsgFunc=MsgFunc_Void;
	RETURN_META(MRES_IGNORED);
}
void MessageBegin(int msg_dest, int msg_type, const float *pOrigin, edict_t *ed)
{
	// ALERT(at_console,"MessageBegin %s\n",GET_USER_MSG_NAME(&Plugin_info,msg_type,0));
	g_msg_type=msg_type;
	g_msg_index=MSGBEGIN;
	MsgFunc=MsgFunc_Void;
	if(msg_type==gmsgCurWeapon){MsgFunc=MsgFunc_CurWeapon;}
	else if(msg_type==gmsgBattery){MsgFunc=MsgFunc_Battery;}
	else if(msg_type==gmsgAmmoX){MsgFunc=MsgFunc_AmmoX;}
	else if(msg_type==gmsgMOTD){MsgFunc=MsgFunc_MOTD;}
	else if(msg_type==gmsgArmorType){MsgFunc=MsgFunc_ArmorType;}
	else if(msg_type==gmsgScoreInfo){MsgFunc=MsgFunc_ScoreInfo;}
	else if(msg_type==gmsgScoreAttrib){MsgFunc=MsgFunc_ScoreAttrib;}
	else if(msg_type==gmsgInitHUD){MsgFunc=MsgFunc_InitHUD;}
	else if(msg_type==gmsgWeaponList){MsgFunc=MsgFunc_WeaponList;}
	if(g_mod_type==CSTRIKE)
	{
		if(msg_type==gmsgTextMsg){MsgFunc=CS_MsgFunc_TextMsg;}
		else if(msg_type==gmsgDeathMsg){MsgFunc=CS_MsgFunc_DeathMsg;}
		else if(msg_type==gmsgScoreInfo){MsgFunc=CS_MsgFunc_ScoreInfo;}
		else if(msg_type==gmsgTeamInfo){MsgFunc=CS_MsgFunc_TeamInfo;}
		else if(msg_type==gmsgScoreAttrib){MsgFunc=CS_MsgFunc_ScoreAttrib;}
		else if(msg_type==gmsgTeamScore){MsgFunc=CS_MsgFunc_TeamScore;}
		else if(msg_type==gmsgSendAudio){MsgFunc=CS_MsgFunc_SendAudio;}
		else if(msg_type==gmsgShowMenu){MsgFunc=CS_MsgFunc_ShowMenu;}
		else if(msg_type==gmsgStatusIcon){MsgFunc=CS_MsgFunc_StatusIcon;}
		else if(msg_type==gmsgInitHUD){MsgFunc=CS_MsgFunc_InitHUD;}
		else if(msg_type==gmsgMoney){MsgFunc=CS_MsgFunc_Money;}
		else if(msg_type==gmsgResetHUD){MsgFunc=CS_MsgFunc_ResetHUD;}
		else if(msg_type==gmsgDamage){MsgFunc=CS_MsgFunc_Damage;}
	}
	if(MsgFunc((void*)ed))
	{
		RETURN_META(MRES_SUPERCEDE);
	}
	else
	{
		RETURN_META(MRES_IGNORED);
	}
}

void WriteByte(int iValue)
{
	g_msg_index++;
	if(MsgFunc((void*)&iValue))
	{
		WRITE_BYTE(iValue);
		RETURN_META(MRES_SUPERCEDE);
	}
	else
	{
		RETURN_META(MRES_IGNORED);
	}
}
void WriteChar(int iValue)
{
	g_msg_index++;
	if(MsgFunc((void*)&iValue))
	{
		WRITE_CHAR(iValue);
		RETURN_META(MRES_SUPERCEDE);
	}
	else
	{
		RETURN_META(MRES_IGNORED);
	}
}
void WriteShort(int iValue)
{
	g_msg_index++;
	if(MsgFunc((void*)&iValue))
	{
		WRITE_SHORT(iValue);
		RETURN_META(MRES_SUPERCEDE);
	}
	else
	{
		RETURN_META(MRES_IGNORED);
	}
}
void WriteLong(int iValue)
{
	g_msg_index++;
	if(MsgFunc((void*)&iValue))
	{
		WRITE_LONG(iValue);
		RETURN_META(MRES_SUPERCEDE);
	}
	else
	{
		RETURN_META(MRES_IGNORED);
	}
}
void WriteAngle(float flValue)
{
	g_msg_index++;
	if(MsgFunc((void*)&flValue))
	{
		WRITE_ANGLE(flValue);
		RETURN_META(MRES_SUPERCEDE);
	}
	else
	{
		RETURN_META(MRES_IGNORED);
	}
}
void WriteCoord(float flValue)
{
	g_msg_index++;
	if(MsgFunc((void*)&flValue))
	{
		WRITE_COORD(flValue);
		RETURN_META(MRES_SUPERCEDE);
	}
	else
	{
		RETURN_META(MRES_IGNORED);
	}
}
void WriteString(const char *sz)
{
	g_msg_index++;
	char string[1025];
	strncpy(string,sz,1024);
	if(MsgFunc((void*)string))
	{
		WRITE_STRING(string);
		RETURN_META(MRES_SUPERCEDE);
	}
	else
	{
		RETURN_META(MRES_IGNORED);
	}
}
void WriteEntity(int iValue)
{
	g_msg_index++;
	if(MsgFunc((void*)&iValue))
	{
		WRITE_ENTITY(iValue);
		RETURN_META(MRES_SUPERCEDE);
	}
	else
	{
		RETURN_META(MRES_IGNORED);
	}
}

void CVarSetFloat(const char *szVarName, float flValue)
{
	if(g_mod_type==CSTRIKE&&strcmp(szVarName,"sv_restartround")==0)
	{
		if(flValue==0)
		{
			CS_Round_Restart();
		}
	}
	RETURN_META(MRES_IGNORED);
}
int RegUserMsg(const char *pszName, int iSize)
{
	int msg_type=META_RESULT_ORIG_RET(int);
	// ALERT(at_console,"pfnRegUserMsg %s %d\n",pszName,msg_type);
	if(strcmp(pszName,"HudText")==0)gmsgHudText=msg_type;
	else if(strcmp(pszName,"TextMsg")==0)gmsgTextMsg=msg_type;
	else if(strcmp(pszName,"SayText")==0)gmsgSayText=msg_type;
	else if(strcmp(pszName,"DeathMsg")==0)gmsgDeathMsg=msg_type;
	else if(strcmp(pszName,"ScoreInfo")==0)gmsgScoreInfo=msg_type;
	else if(strcmp(pszName,"TeamInfo")==0)gmsgTeamInfo=msg_type;
	else if(strcmp(pszName,"ScoreAttrib")==0)gmsgScoreAttrib=msg_type;
	else if(strcmp(pszName,"InitHUD")==0)gmsgInitHUD=msg_type;
	else if(strcmp(pszName,"SendAudio")==0)gmsgSendAudio=msg_type;
	else if(strcmp(pszName,"ShowMenu")==0)gmsgShowMenu=msg_type;
	else if(strcmp(pszName,"VGUIMenu")==0)gmsgVGUIMenu=msg_type;
	else if(strcmp(pszName,"Money")==0)gmsgMoney=msg_type;
	else if(strcmp(pszName,"ResetHUD")==0)gmsgResetHUD=msg_type;
	else if(strcmp(pszName,"TeamScore")==0)gmsgTeamScore=msg_type;
	else if(strcmp(pszName,"CurWeapon")==0)gmsgCurWeapon=msg_type;
	else if(strcmp(pszName,"StatusIcon")==0)gmsgStatusIcon=msg_type;
	else if(strcmp(pszName,"NVGToggle")==0)gmsgNVGToggle=msg_type;
	else if(strcmp(pszName,"Battery")==0)gmsgBattery=msg_type;
	else if(strcmp(pszName,"AmmoX")==0)gmsgAmmoX=msg_type;
	else if(strcmp(pszName,"MOTD")==0)gmsgMOTD=msg_type;
	else if(strcmp(pszName,"Damage")==0)gmsgDamage=msg_type;
	else if(strcmp(pszName,"ArmorType")==0)gmsgArmorType=msg_type;
	else if(strcmp(pszName,"ScreenShake")==0)gmsgScreenShake=msg_type;
	else if(strcmp(pszName,"ScreenFade")==0)gmsgScreenFade=msg_type;
	else if(strcmp(pszName,"WeaponList")==0)gmsgWeaponList=msg_type;
	RETURN_META_VALUE(MRES_IGNORED, META_RESULT_PREVIOUS);
}
void SetClientKeyValue(int clientIndex, char *infobuffer, char *key, char *value)
{
	if(strcmp(key,"model")==0)
	{
		strncpy(g_PlayerInfo[clientIndex].model, value, MAX_MODEL_NAME);
		// SET_MODEL(FakePlayer[clientIndex],UTIL_VarArgs("models/player/%s/%s.mdl", value, value));
	}
	RETURN_META(MRES_IGNORED);
}
void AlertMessage(ALERT_TYPE atype, char *szFmt, ...)
{
	va_list argptr;
	static char msg[1024];
	va_start(argptr, szFmt);
	vsprintf(msg, szFmt,argptr);
	va_end(argptr);
	if(g_mod_type==CSTRIKE)CS_Message(atype, msg);
	RETURN_META(MRES_IGNORED);
}
void FakeClientCommand(edict_t *pEntity, const char *arg1, const char *arg2, const char *arg3)
{
	int length;
	memset(g_argv, 0, sizeof(g_argv));
	isFakeClientCommand=1;
	if((arg1==NULL)||(*arg1==0))
		return;
	if((arg2==NULL)||(*arg2==0))
	{
		length=sprintf(&g_argv[0], "%s", arg1);
		fake_arg_count=1;
	}
	else if((arg3==NULL)||(*arg3==0))
	{
		length=sprintf(&g_argv[0], "%s %s", arg1, arg2);
		fake_arg_count=2;
	}
	else
	{
		length=sprintf(&g_argv[0], "%s %s %s", arg1, arg2, arg3);
		fake_arg_count=3;
	}
	g_argv[length]=0; // null terminate just in case
	strcpy(&g_argv[64], arg1);
	if(arg2)
		strcpy(&g_argv[128], arg2);
	if(arg3)
		strcpy(&g_argv[192], arg3);
	// allow the MOD DLL to execute the ClientCommand...
	(*gpGamedllFuncs->dllapi_table->pfnClientCommand)(pEntity);
	isFakeClientCommand=0;
}
const char *Cmd_Args(void)
{
	if(isFakeClientCommand)
	{
		RETURN_META_VALUE(MRES_SUPERCEDE, &g_argv[0]);
	}
	else
	{
		RETURN_META_VALUE(MRES_IGNORED,(const char *)META_RESULT_PREVIOUS);
	}
}
const char *Cmd_Argv(int argc)
{
	if(isFakeClientCommand)
	{
		if(argc==0)
		{
			RETURN_META_VALUE(MRES_SUPERCEDE, &g_argv[64]);
		}
		else if(argc==1)
		{
			RETURN_META_VALUE(MRES_SUPERCEDE, &g_argv[128]);
		}
		else if(argc==2)
		{
			RETURN_META_VALUE(MRES_SUPERCEDE, &g_argv[192]);
		}
		else
		{
			RETURN_META_VALUE(MRES_SUPERCEDE, NULL);
		}
	}
	else
	{
		RETURN_META_VALUE(MRES_IGNORED,(const char *)META_RESULT_PREVIOUS);
	}
}
int Cmd_Argc(void)
{
	if(isFakeClientCommand)
	{
		RETURN_META_VALUE(MRES_SUPERCEDE, fake_arg_count);
	}
	else
	{
		RETURN_META_VALUE(MRES_IGNORED, META_RESULT_PREVIOUS);
	}
}
void SetPhysicsKeyValue(const edict_t *pClient, const char *key, const char *value)
{
	if(strcmp(key,"hl")==0)
	{
		if(ENTINDEX((edict_t *)pClient)==1)cvar_update();
		Restore((edict_t *)pClient);
	}
	RETURN_META(MRES_IGNORED);
}
void ChangeLevel(char* s1, char* s2)
{
/*	char cmd[256];
	if(UTIL_IsValidEntity(bot))
	{
		sprintf(cmd, "kick \"%s\"\n", STRING(bot->v.netname));
		SERVER_COMMAND(cmd);
		bot=NULL;
	}*/
	CVAR_SET_FLOAT("mp_timelimit",mp_timelimit);
	RETURN_META(MRES_IGNORED);
}
enginefuncs_t meta_engfuncs;
C_DLLEXPORT int GetEngineFunctions(enginefuncs_t *pengfuncsFromEngine, int *interfaceVersion)
{
	LOG_DEVELOPER(PLID, "called: GetEngineFunctions; version=%d", *interfaceVersion);
	if(!pengfuncsFromEngine){
		LOG_ERROR(PLID, "GetEngineFunctions called with null pengfuncsFromEngine");
		return(FALSE);
	}
	else if(*interfaceVersion!=ENGINE_INTERFACE_VERSION){
		LOG_ERROR(PLID, "GetEngineFunctions version mismatch; requested=%d ours=%d", *interfaceVersion, ENGINE_INTERFACE_VERSION);
		// Tell metamod what version we had, so it can figure out who is out of date.
		*interfaceVersion=ENGINE_INTERFACE_VERSION;
		return(FALSE);
	}
	meta_engfuncs.pfnSetModel=SetModel;
	meta_engfuncs.pfnFindEntityByString=FindEntityByString;
	meta_engfuncs.pfnEmitSound=EmitSound;
	meta_engfuncs.pfnMessageBegin=MessageBegin;

	meta_engfuncs.pfnWriteByte=WriteByte;
	meta_engfuncs.pfnWriteChar=WriteChar;
	meta_engfuncs.pfnWriteShort=WriteShort;
	meta_engfuncs.pfnWriteLong=WriteLong;
	meta_engfuncs.pfnWriteAngle=WriteAngle;
	meta_engfuncs.pfnWriteCoord=WriteCoord;
	meta_engfuncs.pfnWriteString=WriteString;
	meta_engfuncs.pfnWriteEntity=WriteEntity;
	meta_engfuncs.pfnCVarSetFloat=CVarSetFloat;
	meta_engfuncs.pfnAlertMessage=AlertMessage;
	meta_engfuncs.pfnCmd_Args=Cmd_Args;
	meta_engfuncs.pfnCmd_Argv=Cmd_Argv;
	meta_engfuncs.pfnCmd_Argc=Cmd_Argc;
	meta_engfuncs.pfnSetClientKeyValue=SetClientKeyValue;
	meta_engfuncs.pfnSetPhysicsKeyValue=SetPhysicsKeyValue;
	meta_engfuncs.pfnChangeLevel=ChangeLevel;
	// meta_engfuncs.pfnDeltaAddEncoder=DeltaAddEncoder;
	memcpy(pengfuncsFromEngine, &meta_engfuncs, sizeof(enginefuncs_t));
	return TRUE;
}
enginefuncs_t meta_engfuncs_post;
C_DLLEXPORT int GetEngineFunctions_Post(enginefuncs_t *pengfuncsFromEngine, int *interfaceVersion)
{
	LOG_DEVELOPER(PLID, "called: GetEngineFunctions; version=%d", *interfaceVersion);
	if(!pengfuncsFromEngine){
		LOG_ERROR(PLID, "GetEngineFunctions called with null pengfuncsFromEngine");
		return(FALSE);
	}
	else if(*interfaceVersion!=ENGINE_INTERFACE_VERSION){
		LOG_ERROR(PLID, "GetEngineFunctions version mismatch; requested=%d ours=%d", *interfaceVersion, ENGINE_INTERFACE_VERSION);
		// Tell metamod what version we had, so it can figure out who is out of date.
		*interfaceVersion=ENGINE_INTERFACE_VERSION;
		return(FALSE);
	}
	meta_engfuncs_post.pfnRegUserMsg=RegUserMsg;
	meta_engfuncs_post.pfnMessageEnd=MessageEnd_Post;
	memcpy(pengfuncsFromEngine, &meta_engfuncs_post, sizeof(enginefuncs_t));
	return TRUE;
}
