// vi: set ts=4 sw=4 :
// vim: set tw=75 :
// Selected portions of dlls/util.cpp from SDK 2.1.
// Functions copied from there as needed...
/***
*
* Copyright(c)1999, 2000 Valve LLC. All rights reserved.
*
* This product contains software technology licensed from Id
* Software, Inc.("Id Technology"). Id Technology(c)1996 Id Software, Inc.
* All Rights Reserved.
*
* Use, distribution, and modification of this source code and/or resulting
* object code is restricted to non-commercial enhancements to products from
* Valve LLC. All other use, distribution, or modification is prohibited
* without written permission from Valve LLC.
*
****/
/*
=====util.cpp========================================================
Utility code. Really not optional after all.
*/
#include <extdll.h>
#include <meta_api.h> // of course
#include <enginecallback.h> // ALERT()
#include "sdk_util.h"
#include "progdefs.h"
#include "edict.h"
#include "const.h"
#include "cbase.h"
#include "shake.h"
#include "weaponinfo.h"
#include "progdefs.h"
#include "scriplib.h"
#include "bmx_plugin.h"

#include "item.h"
#include "usrmsg.h"
#include "playerbuy.h"
#include "moneymsg.h"
//=========================================================
// UTIL_LogPrintf - Prints a logged message to console.
// Preceded by LOG:(timestamp)< message>
//=========================================================
void UTIL_LogPrintf(char *fmt, ...)
{
	va_list argptr;
	static char string[1024];
	va_start(argptr, fmt);
	vsprintf(string, fmt, argptr);
	va_end(argptr);
	// Print to server console
	ALERT(at_logged, "%s", string);
}
float UTIL_AngleDiff(float destAngle, float srcAngle)
{
	float delta;
	delta=destAngle - srcAngle;
	if(destAngle>srcAngle)
	{
		if(delta>=180)
			delta-=360;
	}
	else
	{
		if(delta<=-180)
			delta+=360;
	}
	return delta;
}
Vector UTIL_VecToAngles(const Vector &vec)
{
	float rgflVecOut[3];
	VEC_TO_ANGLES(vec, rgflVecOut);
	return Vector(rgflVecOut);
}
// Overloaded to add IGNORE_GLASS
void UTIL_TraceLine(const Vector &vecStart, const Vector &vecEnd, IGNORE_MONSTERS igmon, IGNORE_GLASS ignoreGlass, edict_t *pentIgnore, TraceResult *ptr)
{
	TRACE_LINE(vecStart, vecEnd,(igmon==ignore_monsters ? TRUE : FALSE)|(ignoreGlass?0x100:0), pentIgnore, ptr);
}
void UTIL_TraceLine(const Vector &vecStart, const Vector &vecEnd, IGNORE_MONSTERS igmon, edict_t *pentIgnore, TraceResult *ptr)
{
	TRACE_LINE(vecStart, vecEnd,(igmon==ignore_monsters ? TRUE : FALSE), pentIgnore, ptr);
}
void UTIL_MakeVectors(const Vector &vecAngles)
{
	MAKE_VECTORS(vecAngles);
}
edict_t *UTIL_FindEntityInSphere(edict_t *pentStart, const Vector &vecCenter, float flRadius)
{
	edict_t *pentEntity;
	pentEntity=FIND_ENTITY_IN_SPHERE(pentStart, vecCenter, flRadius);
	if(!FNullEnt(pentEntity))
		return pentEntity;
	return NULL;
}
edict_t *UTIL_FindEntityByString(edict_t *pentStart, const char *szKeyword, const char *szValue)
{
	edict_t *pentEntity;
	pentEntity=FIND_ENTITY_BY_STRING(pentStart, szKeyword, szValue);
	if(!FNullEnt(pentEntity))
		return pentEntity;
	return NULL;
}
edict_t *UTIL_FindEntityByClassname(edict_t *pentStart, const char *szName)
{
	return UTIL_FindEntityByString(pentStart, "classname", szName);
}
edict_t *UTIL_FindEntityByTargetname(edict_t *pentStart, const char *szName)
{
	return UTIL_FindEntityByString(pentStart, "targetname", szName);
}
// returns a CBaseEntity pointer to a player by index. Only returns if the player is spawned and connected
// otherwise returns NULL
// Index is 1 based
CBaseEntity *UTIL_PlayerByIndex(int playerIndex)
{
	CBaseEntity *pPlayer=NULL;
	if(playerIndex>0 && playerIndex<=gpGlobals->maxClients)
	{
		edict_t *pPlayerEdict=INDEXENT(playerIndex);
		if(pPlayerEdict && !pPlayerEdict->free)
		{
			pPlayer=CBaseEntity::Instance(pPlayerEdict);
		}
	}
	return pPlayer;
}

// Shake the screen of all clients within radius
// radius==0, shake all clients
// UNDONE: Allow caller to shake clients not ONGROUND?
// UNDONE: Fix falloff model(disabled)?
// UNDONE: Affect user controls?
void UTIL_ScreenShake(const Vector &center, float amplitude, float frequency, float duration, float radius)
{
	int i;
	float localAmplitude;
	ScreenShake shake;
	shake.duration=FixedUnsigned16(duration, 1<<12); // 4.12 fixed
	shake.frequency=FixedUnsigned16(frequency, 1<<8); // 8.8 fixed
	for(i=1; i<=gpGlobals->maxClients; i++)
	{
		CBaseEntity *pPlayer=UTIL_PlayerByIndex(i);
		if(!pPlayer || !(pPlayer->pev->flags & FL_ONGROUND))// Don't shake if not onground
			continue;
		localAmplitude=0;
		if(radius<=0)
			localAmplitude=amplitude;
		else
		{
			Vector delta=center - pPlayer->pev->origin;
			float distance=delta.Length();
			// Had to get rid of this falloff - it didn't work well
			if(distance<radius)
				localAmplitude=amplitude;//radius - distance;
		}
		if(localAmplitude)
		{
			shake.amplitude=FixedUnsigned16(localAmplitude, 1<<12); // 4.12 fixed
			MESSAGE_BEGIN(MSG_ONE, gmsgScreenShake, NULL, pPlayer->edict()); // use the magic #1 for "one client"
				WRITE_SHORT(shake.amplitude); // shake amount
				WRITE_SHORT(shake.duration); // shake lasts this long
				WRITE_SHORT(shake.frequency); // shake noise frequency
			MESSAGE_END();
		}
	}
}
void UTIL_ScreenShakeAll(const Vector &center, float amplitude, float frequency, float duration)
{
	UTIL_ScreenShake(center, amplitude, frequency, duration, 0);
}
void UTIL_ScreenFadeBuild(ScreenFade &fade, const Vector &color, float fadeTime, float fadeHold, int alpha, int flags)
{
	fade.duration=FixedUnsigned16(fadeTime, 1<<12); // 4.12 fixed
	fade.holdTime=FixedUnsigned16(fadeHold, 1<<12); // 4.12 fixed
	fade.r=(int)color.x;
	fade.g=(int)color.y;
	fade.b=(int)color.z;
	fade.a=alpha;
	fade.fadeFlags=flags;
}
void UTIL_ScreenFadeWrite(const ScreenFade &fade, CBaseEntity *pEntity)
{
	if(!pEntity || !pEntity->IsNetClient())
		return;
	MESSAGE_BEGIN(MSG_ONE, gmsgScreenFade, NULL, pEntity->edict()); // use the magic #1 for "one client"
		WRITE_SHORT(fade.duration); // fade lasts this long
		WRITE_SHORT(fade.holdTime); // fade lasts this long
		WRITE_SHORT(fade.fadeFlags); // fade type(in / out)
		WRITE_BYTE(fade.r); // fade red
		WRITE_BYTE(fade.g); // fade green
		WRITE_BYTE(fade.b); // fade blue
		WRITE_BYTE(fade.a); // fade blue
	MESSAGE_END();
}
void UTIL_ScreenFadeAll(const Vector &color, float fadeTime, float fadeHold, int alpha, int flags)
{
	int i;
	ScreenFade fade;
	UTIL_ScreenFadeBuild(fade, color, fadeTime, fadeHold, alpha, flags);
	for(i=1; i<=gpGlobals->maxClients; i++)
	{
		CBaseEntity *pPlayer=UTIL_PlayerByIndex(i);
		UTIL_ScreenFadeWrite(fade, pPlayer);
	}
}
void UTIL_ScreenFade(CBaseEntity *pEntity, const Vector &color, float fadeTime, float fadeHold, int alpha, int flags)
{
	ScreenFade fade;
	UTIL_ScreenFadeBuild(fade, color, fadeTime, fadeHold, alpha, flags);
	UTIL_ScreenFadeWrite(fade, pEntity);
}
void UTIL_HudMessage(edict_t *pEdict, const hudtextparms_t &textparms, const char *pMessage)
{
	if(!UTIL_IsValidEntity(pEdict))
		return;
	MESSAGE_BEGIN(MSG_ONE, SVC_TEMPENTITY, NULL, pEdict);
		WRITE_BYTE(TE_TEXTMESSAGE);
		WRITE_BYTE(textparms.channel & 0xFF);
		WRITE_SHORT(FixedSigned16(textparms.x, 1<<13));
		WRITE_SHORT(FixedSigned16(textparms.y, 1<<13));
		WRITE_BYTE(textparms.effect);
		WRITE_BYTE(textparms.r1);
		WRITE_BYTE(textparms.g1);
		WRITE_BYTE(textparms.b1);
		WRITE_BYTE(textparms.a1);
		WRITE_BYTE(textparms.r2);
		WRITE_BYTE(textparms.g2);
		WRITE_BYTE(textparms.b2);
		WRITE_BYTE(textparms.a2);
		WRITE_SHORT(FixedUnsigned16(textparms.fadeinTime, 1<<8));
		WRITE_SHORT(FixedUnsigned16(textparms.fadeoutTime, 1<<8));
		WRITE_SHORT(FixedUnsigned16(textparms.holdTime, 1<<8));
	if(textparms.effect==2)
		WRITE_SHORT(FixedUnsigned16(textparms.fxTime, 1<<8));
	if(strlen(pMessage)< 512)
	{
		WRITE_STRING(pMessage);
	}
	else
	{
		char tmp[512];
		strncpy(tmp, pMessage, 511);
		tmp[511]=0;
		WRITE_STRING(tmp);
	}
	MESSAGE_END();
}
void UTIL_HudMessageAll(const hudtextparms_t &textparms, const char *pMessage)
{
	int i;
	for(i=1; i<=gpGlobals->maxClients; i++)
	{
		UTIL_HudMessage(INDEXENT(i), textparms, pMessage);
	}
}
void UTIL_ClientPrintAll(int msg_dest, const char *msg_name, const char *param1, const char *param2, const char *param3, const char *param4)
{
	MESSAGE_BEGIN(MSG_ALL, gmsgTextMsg);
		WRITE_BYTE(msg_dest);
		WRITE_STRING(msg_name);
	if(param1)
		WRITE_STRING(param1);
	if(param2)
		WRITE_STRING(param2);
	if(param3)
		WRITE_STRING(param3);
	if(param4)
		WRITE_STRING(param4);
	MESSAGE_END();
}
void ClientPrint(edict_t *pEntity, int msg_dest, const char *msg_name, const char *param1, const char *param2, const char *param3, const char *param4)
{
	MESSAGE_BEGIN(MSG_ONE, gmsgTextMsg, NULL, pEntity);
		WRITE_BYTE(msg_dest);
		WRITE_STRING(msg_name);
	if(param1)
		WRITE_STRING(param1);
	if(param2)
		WRITE_STRING(param2);
	if(param3)
		WRITE_STRING(param3);
	if(param4)
		WRITE_STRING(param4);
	MESSAGE_END();
}
void UTIL_SayText(const char *pText, edict_t *pEdict)
{
	if(!UTIL_IsValidEntity(pEdict))
		return;
	MESSAGE_BEGIN(MSG_ONE, gmsgSayText, NULL, pEdict);
		WRITE_BYTE(ENTINDEX(pEdict));
		WRITE_STRING(pText);
	MESSAGE_END();
}
void UTIL_SayTextAll(const char *pText, edict_t *pEdict)
{
	if(!UTIL_IsValidEntity(pEdict))
		return;
	MESSAGE_BEGIN(MSG_ALL, gmsgSayText, NULL);
		WRITE_BYTE(ENTINDEX(pEdict));
		WRITE_STRING(pText);
	MESSAGE_END();
}
void UTIL_SayTextAll(const char *pText)
{
	for(int i=1; i<=MAX_PLAYERS; i++)
	{
		edict_t *pEdict=INDEXENT(i);
		if(UTIL_IsValidEntity(pEdict)&& !IsBot(pEdict))
		{
			MESSAGE_BEGIN(MSG_ONE, gmsgSayText, NULL,pEdict);
				WRITE_BYTE(0);
				WRITE_STRING(pText);
			MESSAGE_END();
		}
	}
}
void UTIL_ShowMessage(const char *pString, edict_t *pEdict)
{
	if(!UTIL_IsValidEntity(pEdict))
		return;
	MESSAGE_BEGIN(MSG_ONE, gmsgHudText, NULL, pEdict);
		WRITE_STRING(pString);
	MESSAGE_END();
}
void UTIL_ShowMessageAll(const char *pString)
{
	int i;
	// loop through all players
	for(i=1; i<=gpGlobals->maxClients; i++)
	{
		UTIL_ShowMessage(pString, INDEXENT(i));
	}
}
BOOL UTIL_IsValidEntity(edict_t *pent)
{
	if(!pent || pent->free ||(pent->v.flags & FL_KILLME)||!(pent->v.flags & FL_CLIENT))
		return FALSE;
	return TRUE;
}
BOOL IsAlive(edict_t *pEdict)
{
	return((pEdict->v.deadflag==DEAD_NO)&&
		(pEdict->v.health>0)&& !(pEdict->v.flags & FL_NOTARGET));
}
void UTIL_ShowMenu(edict_t *pEdict, int slots, int displaytime, bool needmore, char *pText)
{
	if(gmsgShowMenu==0)
		gmsgShowMenu=REG_USER_MSG("ShowMenu", -1);
	MESSAGE_BEGIN(MSG_ONE, gmsgShowMenu, NULL, pEdict);
		WRITE_SHORT(slots);
		WRITE_CHAR(displaytime);
		WRITE_BYTE(needmore);
		WRITE_STRING(pText);
	MESSAGE_END();
}
bool IsBot(edict_t *pEdict)
{
	if(pEdict)
	{
		if(pEdict->v.flags & FL_FAKECLIENT)
			return true;
	}
	return false;
}
#include <stdarg.h>
#include <string.h>
char* UTIL_VarArgs(char *format, ...)
{
	va_list argptr;
	static char string[1024];
	va_start(argptr, format);
	vsprintf(string, format,argptr);
	va_end(argptr);
	return string;
}
void UTIL_StringToVector(float *pVector, const char *pString)
{
	char *pstr, *pfront, tempString[128];
	int j;
	strcpy(tempString, pString);
	pstr=pfront=tempString;
	for(j=0; j<3; j++)// lifted from pr_edict.c
	{
		pVector[j]=atof(pfront);
		while(*pstr && *pstr!=' ')
			pstr++;
		if(!*pstr)
			break;
		pstr++;
		pfront=pstr;
	}
	if(j<2)
	{
	/*
	ALERT(at_error, "Bad field in entity!! %s:%s==\"%s\"\n",
	pkvd->szClassName, pkvd->szKeyName, pkvd->szValue);
		*/
		for(j=j+1;j<3; j++)
			pVector[j]=0;
	}
}
